/**
 * Copyright (c) 2010-2011, Chair for Communication and Distributed Systems,
 * RWTH Aachen University, Germany
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * @author Bernhard Kirchen
 */

#ifndef __TINYWIFIERROR_H
#define __TINYWIFIERROR_H

#include "tinyWifiHeaders.h"

enum {
  ERR_NOTICE = 0,
  ERR_GENERAL = -1,
  ERR_TIMER = -2,
  ERR_INIT = -3,
  ERR_NET = -4
};

static pthread_mutex_t mutex_errorMessage;

void tinyWifiNetError ( int status, const char *path, const char *function,
                        int line, const char *message, int net_result ) {
  char* file;

  pthread_mutex_lock(&mutex_errorMessage);
    fflush(stdout);
    file = basename((char*)path);

    fprintf(stderr, "%s: In function '%s':\n", file, function);
    if ( status == ERR_NOTICE ) {
      fprintf(stderr, "%s:%i: WARNING: %s\n", file, line, message);
    }
    else {
      fprintf(stderr, "%s:%i: ERROR: %s\n", file, line, message);
    }
    if ( errno != 0 ) {
      fprintf(stderr, "%s:%i: ", file, line);
      perror("Linux says it was (errno)");
    }
    if ( net_result != 0 ) {
      fprintf(stderr, "%s:%i: gai_strerror says it was: %s\n", file, line,
          gai_strerror(net_result));
    }
    if (status != ERR_NOTICE) {
      fprintf(stderr, "%s:%i: program exits with status %i.\n", file, line,
          status);
      exit(status);
    }
  pthread_mutex_unlock(&mutex_errorMessage);
}

void tinyWifiError ( int status, const char *file, const char *function,
                     int line, const char *message ) {
  tinyWifiNetError(status, file, function, line, message, 0);
}

#endif
