/**
 * Copyright (c) 2010-2011, Chair for Communication and Distributed Systems,
 * RWTH Aachen University, Germany
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * Defines message structures for the TinyWifi platform. Also sets some
 * important constants related to communication.
 *
 * @author Bernhard Kirchen
 */

#ifndef PLATFORM_MESSAGE_H
#define PLATFORM_MESSAGE_H

//size of the radio active message receiver buffer [#messages]
#ifndef TINIWIFI_RECVBUF
#define TINIWIFI_RECVBUF 5
#endif

//size of the serial message receiver buffer [#bytes]
//should be set generously, must fit at least one serial message!
#ifndef TINIWIFI_RECVBUF_SERIAL
#define TINIWIFI_RECVBUF_SERIAL 512
#endif

//duration of acknowledgement timeout [binary ms]
#ifndef TINIWIFI_ACK_TIMEOUT
#define TINIWIFI_ACK_TIMEOUT 256
#endif

//we adopt the existing serial message style
#include "Serial.h"

enum ack_status {
  ACK_NONE = 0,
  ACK_REQUESTED = 1,
  ACK_ACKED = 2
};

/**
 * WiFi message stuff
 */
typedef nx_struct wifi_header {
  nx_am_addr_t dest;
  nx_am_addr_t src;
  nx_uint8_t length;
  nx_am_group_t group;
  nx_am_id_t type;
  nx_uint8_t ack;
  nx_uint32_t identifier;
} wifi_header_t;

typedef nx_struct wifi_footer {
} wifi_footer_t;

typedef nx_struct wifi_metadata {
  nx_uint32_t time32khz;
  nx_uint32_t timeMilli;
  nx_uint8_t time32khz_valid;
  nx_uint8_t timeMilli_valid;
  nx_uint8_t wasAcked;
} wifi_metadata_t;

/**
 * Unions of message headers, footers and metadata
 */
typedef union message_header {
  wifi_header_t wifi_header;
  serial_header_t serial_header;
} message_header_t;

typedef union message_footer {
  wifi_footer_t wifi_footer;
} message_footer_t;

typedef union message_metadata {
  wifi_metadata_t wifi_metadata;
  serial_metadata_t serial_metadata;
} message_metadata_t;

#endif
